import bpy, os


# 設定data.blend檔案的路徑
library_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "Library/DATA/data.blend")




#將節點組導入到Blender文件中的函數
def node_import(shader_name, library_path):
    # 檢查節點組是否已存在於當前資料中
    if shader_name in bpy.data.node_groups:
        print(f"節點組 '{shader_name}' 已存在，將不再重複導入")
        return True

    try:
        with bpy.data.libraries.load(library_path, link=False) as (data_from, data_to):
            if shader_name in data_from.node_groups:
                data_to.node_groups.append(shader_name)
            else:
                print(f"節點組 '{shader_name}' 未找到")
                return False
    except Exception as e:
        print(f"導入節點組時發生錯誤: {e}")
        return False

    print(f"節點組 '{shader_name}' 已成功導入並應用")
    return True


def load_node_groups_from_blend(blend_path):
    with bpy.data.libraries.load(blend_path) as (data_from, data_to):
        data_to.node_groups = data_from.node_groups
    return [ng.name for ng in data_to.node_groups]


# 將幾何節點組應用到所選Mesh對象的函數
def apply_geometry_nodes_to_selected_mesh(shader_name):
    # 从 Blender 的数据库中获取指定的节点组
    node_group = bpy.data.node_groups.get(shader_name)
    if not node_group:
        print(f"未找到名为 '{shader_name}' 的节点组。")
        return False

    # 遍历选中对象中的每个 Mesh 类型对象和 Curve 类型对象
    for obj in bpy.context.selected_objects:
        if obj.type in ['MESH', 'CURVE']:
            # 为对象添加并命名 Geometry Nodes 修改器
            mod = obj.modifiers.new(name=shader_name, type='NODES')
            if mod:
                # 设置修改器使用的节点组
                mod.node_group = node_group
                print(f"已为对象 '{obj.name}' 添加名为 '{shader_name}' 的 Geometry Nodes 修改器，并设置了节点组。")
            else:
                print(f"无法为对象 '{obj.name}' 添加修改器。")

    return True


# 將節點組應用到所選Mesh對象的函數
def apply_node_group_to_selected_mesh(shader_name):
    # 从data.blend文件中加载所有节点组名称
    all_shader_names = load_node_groups_from_blend(library_path)

    # 检查所选Mesh对象的材质中是否已存在列表中的任何一个shader_name
    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH':
            for slot in obj.material_slots:
                material = slot.material
                if material and material.use_nodes:
                    nodes = material.node_tree.nodes
                    for existing_shader_name in all_shader_names:
                        if any(node for node in nodes if node.type == 'GROUP' and node.node_tree and node.node_tree.name == existing_shader_name):
                            print(f"节点组 '{existing_shader_name}' 已存在于材质中，跳过添加。")
                            return  # 这里考虑是否应继续而不是返回，视需求而定

    # 如果没有找到任何一个shader_name节点组，继续执行添加操作
    for obj in bpy.context.selected_objects:
        if obj.type != 'MESH':
            continue

        for slot in obj.material_slots:
            if slot.material:
                mat = slot.material
                if not mat:
                    mat = bpy.data.materials.new(name="NodeGroupMaterial")
                    try:
                        mat.use_nodes = True
                    except AttributeError as e:
                        print("Error: ", e)

                mat.use_nodes = True
                nodes = mat.node_tree.nodes
                links = mat.node_tree.links

                # 检查是否已经存在指定的shader_name节点组
                existing_node_group = next((node for node in nodes if node.type == 'GROUP' and node.node_tree and node.node_tree.name == shader_name), None)
                if existing_node_group:
                    print(f"节点组 '{shader_name}' 已存在于材质中，跳过添加。")
                    continue  # 跳过当前插槽，继续处理下一个

                material_output = next((node for node in nodes if node.type == 'OUTPUT_MATERIAL'), None)
                if material_output and material_output.inputs['Surface'].is_linked:
                    original_link = material_output.inputs['Surface'].links[0]
                    from_node = original_link.from_node
                    from_socket = original_link.from_socket
                    links.remove(original_link)
                
                # 检查是否已经存在名为 '19_sfgw89_Reroute_us64_001' 和 '19_sfgw89_Reroute_us64_002' 的节点
                reroute1 = nodes.get('19_sfgw89_Reroute_us64_001')
                if not reroute1:
                    reroute1 = nodes.new(type='NodeReroute')
                    reroute1.name = '19_sfgw89_Reroute_us64_001'
                    reroute1["NodeFX_Tag"] = True
                reroute2 = nodes.get('19_sfgw89_Reroute_us64_002')
                if not reroute2:
                    reroute2 = nodes.new(type='NodeReroute')
                    reroute2.name = '19_sfgw89_Reroute_us64_002'
                    reroute2["NodeFX_Tag"] = True
                links.new(from_socket, reroute1.inputs[0])
                links.new(reroute1.outputs[0], reroute2.inputs[0])

                # 将节点组实例添加到材质节点树中
                node_group_instance = nodes.new('ShaderNodeGroup')
                node_group_instance.node_tree = bpy.data.node_groups.get(shader_name)
                node_group_instance["NodeFX_Tag"] = True
                if not node_group_instance.node_tree:
                    print(f"节点组 '{shader_name}' 未找到。")
                    continue

                if not material_output:
                    material_output = nodes.new(type='ShaderNodeOutputMaterial')

                if 'Surface' in node_group_instance.outputs:
                    links.new(node_group_instance.outputs['Surface'], material_output.inputs['Surface'])
                else:
                    print(f"节点组 '{shader_name}' 没有名为'Surface'的输出。")
                    continue

                if 'reroute2' in locals():
                    links.new(reroute2.outputs[0], material_output.inputs['Surface'])

                print(f"节点组 '{shader_name}' 已成功应用到Mesh对象：{obj.name}的材质插槽 {slot.name}。")
            else:
                pass
    return True
